@@ -354,8 +354,12 @@ HWY_API Vec128<T, N> IfThenZeroElse(Mask128<T, N> mask, Vec128<T, N> no) {
 template <typename T, size_t N>
 HWY_API Vec128<T, N> IfNegativeThenElse(Vec128<T, N> v, Vec128<T, N> yes,
                                         Vec128<T, N> no) {
+  const DFromV<decltype(v)> d;
+  const RebindToSigned<decltype(d)> di;
+  const auto vi = BitCast(di, v);
+
   for (size_t i = 0; i < N; ++i) {
-    v.raw[i] = v.raw[i] < 0 ? yes.raw[i] : no.raw[i];
+    v.raw[i] = vi.raw[i] < 0 ? yes.raw[i] : no.raw[i];
   }
   return v;
 }
@@ -854,50 +858,37 @@ template <size_t N>
 HWY_API Vec128<int16_t, N> MulFixedPoint15(Vec128<int16_t, N> a,
                                            Vec128<int16_t, N> b) {
   for (size_t i = 0; i < N; ++i) {
-    a.raw[i] = static_cast<int16_t>((2 * a.raw[i] * b.raw[i] + 32768) >> 16);
+    a.raw[i] = static_cast<int16_t>((a.raw[i] * b.raw[i] + 16384) >> 15);
   }
   return a;
 }
 
-// Multiplies even lanes (0, 2 ..) and returns the double-wide result.
-template <size_t N>
-HWY_API Vec128<int64_t, (N + 1) / 2> MulEven(Vec128<int32_t, N> a,
-                                             Vec128<int32_t, N> b) {
-  Vec128<int64_t, (N + 1) / 2> mul;
+// Multiplies even lanes (0, 2, ..) and returns the double-wide result.
+template <class T, size_t N,
+          HWY_IF_T_SIZE_ONE_OF(T, (1 << 1) | (1 << 2) | (1 << 4)),
+          HWY_IF_NOT_FLOAT_NOR_SPECIAL(T)>
+HWY_API Vec128<MakeWide<T>, (N + 1) / 2> MulEven(Vec128<T, N> a,
+                                                 Vec128<T, N> b) {
+  using TW = MakeWide<T>;
+  Vec128<TW, (N + 1) / 2> mul;
   for (size_t i = 0; i < N; i += 2) {
-    const int64_t a64 = a.raw[i];
-    mul.raw[i / 2] = a64 * b.raw[i];
-  }
-  return mul;
-}
-template <size_t N>
-HWY_API Vec128<uint64_t, (N + 1) / 2> MulEven(Vec128<uint32_t, N> a,
-                                              Vec128<uint32_t, N> b) {
-  Vec128<uint64_t, (N + 1) / 2> mul;
-  for (size_t i = 0; i < N; i += 2) {
-    const uint64_t a64 = a.raw[i];
-    mul.raw[i / 2] = a64 * b.raw[i];
+    const TW a_wide = a.raw[i];
+    mul.raw[i / 2] = static_cast<TW>(a_wide * b.raw[i]);
   }
   return mul;
 }
 
-template <size_t N>
-HWY_API Vec128<int64_t, (N + 1) / 2> MulOdd(Vec128<int32_t, N> a,
-                                            Vec128<int32_t, N> b) {
-  Vec128<int64_t, (N + 1) / 2> mul;
+// Multiplies odd lanes (1, 3, ..) and returns the double-wide result.
+template <class T, size_t N,
+          HWY_IF_T_SIZE_ONE_OF(T, (1 << 1) | (1 << 2) | (1 << 4)),
+          HWY_IF_NOT_FLOAT_NOR_SPECIAL(T)>
+HWY_API Vec128<MakeWide<T>, (N + 1) / 2> MulOdd(Vec128<T, N> a,
+                                                Vec128<T, N> b) {
+  using TW = MakeWide<T>;
+  Vec128<TW, (N + 1) / 2> mul;
   for (size_t i = 0; i < N; i += 2) {
-    const int64_t a64 = a.raw[i + 1];
-    mul.raw[i / 2] = a64 * b.raw[i + 1];
-  }
-  return mul;
-}
-template <size_t N>
-HWY_API Vec128<uint64_t, (N + 1) / 2> MulOdd(Vec128<uint32_t, N> a,
-                                             Vec128<uint32_t, N> b) {
-  Vec128<uint64_t, (N + 1) / 2> mul;
-  for (size_t i = 0; i < N; i += 2) {
-    const uint64_t a64 = a.raw[i + 1];
-    mul.raw[i / 2] = a64 * b.raw[i + 1];
+    const TW a_wide = a.raw[i + 1];
+    mul.raw[i / 2] = static_cast<TW>(a_wide * b.raw[i + 1]);
   }
   return mul;
 }
@@ -913,8 +904,9 @@ HWY_API Vec128<float, N> ApproximateReciprocal(Vec128<float, N> v) {
   return v;
 }
 
-template <size_t N>
-HWY_API Vec128<float, N> AbsDiff(Vec128<float, N> a, Vec128<float, N> b) {
+// generic_ops takes care of integer T.
+template <typename T, size_t N, HWY_IF_FLOAT(T)>
+HWY_API Vec128<T, N> AbsDiff(Vec128<T, N> a, Vec128<T, N> b) {
   return Abs(a - b);
 }
 
@@ -1347,6 +1339,22 @@ HWY_API VFromD<D> LoadDup128(D d, const TFromD<D>* HWY_RESTRICT aligned) {
   return Load(d, aligned);
 }
 
+#ifdef HWY_NATIVE_LOAD_N
+#undef HWY_NATIVE_LOAD_N
+#else
+#define HWY_NATIVE_LOAD_N
+#endif
+
+template <class D>
+HWY_API VFromD<D> LoadN(D d, const TFromD<D>* HWY_RESTRICT p,
+                        size_t max_lanes_to_load) {
+  VFromD<D> v = Zero(d);
+  const size_t N = Lanes(d);
+  const size_t num_of_lanes_to_load = HWY_MIN(max_lanes_to_load, N);
+  CopyBytes(p, v.raw, num_of_lanes_to_load * sizeof(TFromD<D>));
+  return v;
+}
+
 // ------------------------------ Store
 
 template <class D>
@@ -1367,6 +1375,20 @@ HWY_API void BlendedStore(VFromD<D> v, MFromD<D> m, D d,
   }
 }
 
+#ifdef HWY_NATIVE_STORE_N
+#undef HWY_NATIVE_STORE_N
+#else
+#define HWY_NATIVE_STORE_N
+#endif
+
+template <class D>
+HWY_API void StoreN(VFromD<D> v, D d, TFromD<D>* HWY_RESTRICT p,
+                    size_t max_lanes_to_store) {
+  const size_t N = Lanes(d);
+  const size_t num_of_lanes_to_store = HWY_MIN(max_lanes_to_store, N);
+  CopyBytes(v.raw, p, num_of_lanes_to_store * sizeof(TFromD<D>));
+}
+
 // ------------------------------ LoadInterleaved2/3/4
 
 // Per-target flag to prevent generic_ops-inl.h from defining LoadInterleaved2.
@@ -1466,52 +1488,8 @@ HWY_API void Stream(VFromD<D> v, D d, TFromD<D>* HWY_RESTRICT aligned) {
   Store(v, d, aligned);
 }
 
-// ------------------------------ Scatter
-
-template <class D, typename T = TFromD<D>, typename Offset>
-HWY_API void ScatterOffset(VFromD<D> v, D d, T* base,
-                           Vec128<Offset, HWY_MAX_LANES_D(D)> offset) {
-  static_assert(sizeof(T) == sizeof(Offset), "Index/lane size must match");
-  for (size_t i = 0; i < MaxLanes(d); ++i) {
-    uint8_t* const base8 = reinterpret_cast<uint8_t*>(base) + offset.raw[i];
-    CopyBytes<sizeof(T)>(&v.raw[i], base8);  // copy to bytes
-  }
-}
-
-template <class D, typename T = TFromD<D>, typename Index>
-HWY_API void ScatterIndex(VFromD<D> v, D d, T* HWY_RESTRICT base,
-                          Vec128<Index, HWY_MAX_LANES_D(D)> index) {
-  static_assert(sizeof(T) == sizeof(Index), "Index/lane size must match");
-  for (size_t i = 0; i < MaxLanes(d); ++i) {
-    base[index.raw[i]] = v.raw[i];
-  }
-}
-
-// ------------------------------ Gather
-
-template <class D, typename T = TFromD<D>, typename Offset>
-HWY_API VFromD<D> GatherOffset(D d, const T* base,
-                               Vec128<Offset, HWY_MAX_LANES_D(D)> offset) {
-  static_assert(sizeof(T) == sizeof(Offset), "Index/lane size must match");
-  VFromD<D> v;
-  for (size_t i = 0; i < MaxLanes(d); ++i) {
-    const uint8_t* base8 =
-        reinterpret_cast<const uint8_t*>(base) + offset.raw[i];
-    CopyBytes<sizeof(T)>(base8, &v.raw[i]);  // copy from bytes
-  }
-  return v;
-}
-
-template <class D, typename T = TFromD<D>, typename Index>
-HWY_API VFromD<D> GatherIndex(D d, const T* HWY_RESTRICT base,
-                              Vec128<Index, HWY_MAX_LANES_D(D)> index) {
-  static_assert(sizeof(T) == sizeof(Index), "Index/lane size must match");
-  VFromD<D> v;
-  for (size_t i = 0; i < MaxLanes(d); ++i) {
-    v.raw[i] = base[index.raw[i]];
-  }
-  return v;
-}
+// ------------------------------ Scatter in generic_ops-inl.h
+// ------------------------------ Gather in generic_ops-inl.h
 
 // ================================================== CONVERT
 
@@ -2166,7 +2144,8 @@ HWY_API VFromD<D> SlideUpLanes(D d, VFromD<D> v, size_t amt) {
   VFromD<D> ret = Zero(d);
   constexpr size_t N = HWY_MAX_LANES_D(D);
   const size_t clamped_amt = HWY_MIN(amt, N);
-  memcpy(ret.raw + clamped_amt, v.raw, (N - clamped_amt) * sizeof(TFromD<D>));
+  CopyBytes(v.raw, ret.raw + clamped_amt,
+            (N - clamped_amt) * sizeof(TFromD<D>));
   return ret;
 }
 
@@ -2177,7 +2156,8 @@ HWY_API VFromD<D> SlideDownLanes(D d, VFromD<D> v, size_t amt) {
   VFromD<D> ret = Zero(d);
   constexpr size_t N = HWY_MAX_LANES_D(D);
   const size_t clamped_amt = HWY_MIN(amt, N);
-  memcpy(ret.raw, v.raw + clamped_amt, (N - clamped_amt) * sizeof(TFromD<D>));
+  CopyBytes(v.raw + clamped_amt, ret.raw,
+            (N - clamped_amt) * sizeof(TFromD<D>));
   return ret;
 }
 
@@ -2542,7 +2522,9 @@ HWY_API size_t CompressBitsStore(VFromD<D> v, const uint8_t* HWY_RESTRICT bits,
 
 // ------------------------------ Additional mask logical operations
 template <class T>
-HWY_API Mask128<T, 1> SetAtOrAfterFirst(Mask128<T, 1> mask) { return mask; }
+HWY_API Mask128<T, 1> SetAtOrAfterFirst(Mask128<T, 1> mask) {
+  return mask;
+}
 
 template <class T, size_t N, HWY_IF_LANES_GT(N, 1)>
 HWY_API Mask128<T, N> SetAtOrAfterFirst(Mask128<T, N> mask) {
@@ -2622,6 +2604,19 @@ HWY_API VFromD<D> WidenMulPairwiseAdd(D d32, VI16 a, VI16 b) {
   return Add(Mul(ae, be), Mul(ao, bo));
 }
 
+template <class D, HWY_IF_U32_D(D), class VU16>
+HWY_API VFromD<D> WidenMulPairwiseAdd(D du32, VU16 a, VU16 b) {
+  const auto lo16_mask = Set(du32, 0x0000FFFFu);
+
+  const auto a0 = And(BitCast(du32, a), lo16_mask);
+  const auto b0 = And(BitCast(du32, b), lo16_mask);
+
+  const auto a1 = ShiftRight<16>(BitCast(du32, a));
+  const auto b1 = ShiftRight<16>(BitCast(du32, b));
+
+  return Add(Mul(a0, b0), Mul(a1, b1));
+}
+
 // ------------------------------ ReorderWidenMulAccumulate (MulAdd, ZipLower)
 
 template <class D, HWY_IF_F32_D(D), size_t N, class VBF16>
@@ -2654,6 +2649,20 @@ HWY_API VFromD<D> ReorderWidenMulAccumulate(D d32, VI16 a, VI16 b,
   return Add(Mul(ae, be), sum0);
 }
 
+template <class D, HWY_IF_U32_D(D), size_t N, class VU16>
+HWY_API VFromD<D> ReorderWidenMulAccumulate(D du32, VU16 a, VU16 b,
+                                            const Vec128<uint32_t, N> sum0,
+                                            Vec128<uint32_t, N>& sum1) {
+  using VU32 = VFromD<decltype(du32)>;
+  const VU32 lo16_mask = Set(du32, uint32_t{0x0000FFFFu});
+  const VU32 ae = And(BitCast(du32, a), lo16_mask);
+  const VU32 be = And(BitCast(du32, b), lo16_mask);
+  const VU32 ao = ShiftRight<16>(BitCast(du32, a));
+  const VU32 bo = ShiftRight<16>(BitCast(du32, b));
+  sum1 = Add(Mul(ao, bo), sum1);
+  return Add(Mul(ae, be), sum0);
+}
+
 // ------------------------------ RearrangeToOddPlusEven
 template <class VW>
 HWY_API VW RearrangeToOddPlusEven(VW sum0, VW sum1) {
