@@ -16,6 +16,8 @@
 #include <stdio.h>
 #include <string.h>  // memcmp
 
+#include "hwy/nanobenchmark.h"
+
 #undef HWY_TARGET_INCLUDE
 #define HWY_TARGET_INCLUDE "tests/mask_mem_test.cc"
 #include "hwy/foreach_target.h"  // IWYU pragma: keep
@@ -64,6 +66,87 @@ HWY_NOINLINE void TestAllMaskedLoad() {
   ForAllTypes(ForPartialVectors<TestMaskedLoad>());
 }
 
+struct TestMaskedScatter {
+  template <class T, class D>
+  HWY_NOINLINE void operator()(T /*unused*/, D d) {
+    RandomState rng;
+
+    using TI = MakeSigned<T>;  // For mask > 0 comparison
+    const Rebind<TI, D> di;
+    using VI = Vec<decltype(di)>;
+    const size_t N = Lanes(d);
+    auto bool_lanes = AllocateAligned<TI>(N);
+    auto lanes = AllocateAligned<T>(N);
+    auto expected = AllocateAligned<T>(N);
+    HWY_ASSERT(bool_lanes && lanes && expected);
+
+    const Vec<D> v = Iota(d, static_cast<T>(hwy::Unpredictable1() - 1));
+    Store(v, d, lanes.get());
+
+    const VI indices =
+        Reverse(di, Iota(di, static_cast<TI>(hwy::Unpredictable1() - 1)));
+
+    // Each lane should have a chance of having mask=true.
+    for (size_t rep = 0; rep < AdjustedReps(200); ++rep) {
+      ZeroBytes(expected.get(), N * sizeof(T));
+      for (size_t i = 0; i < N; ++i) {
+        bool_lanes[i] = (Random32(&rng) & 1024) ? TI(1) : TI(0);
+        if (bool_lanes[i]) {
+          expected[N - 1 - i] = static_cast<T>(i);
+        }
+      }
+
+      const VI mask_i = Load(di, bool_lanes.get());
+      const auto mask = RebindMask(d, Gt(mask_i, Zero(di)));
+      ZeroBytes(lanes.get(), N * sizeof(T));
+      MaskedScatterIndex(v, mask, d, lanes.get(), indices);
+      HWY_ASSERT_VEC_EQ(d, expected.get(), Load(d, lanes.get()));
+    }
+  }
+};
+
+HWY_NOINLINE void TestAllMaskedScatter() {
+  ForUIF3264(ForPartialVectors<TestMaskedScatter>());
+}
+
+struct TestMaskedGather {
+  template <class T, class D>
+  HWY_NOINLINE void operator()(T /*unused*/, D d) {
+    RandomState rng;
+
+    using TI = MakeSigned<T>;  // For mask > 0 comparison
+    const Rebind<TI, D> di;
+    using VI = Vec<decltype(di)>;
+    const size_t N = Lanes(d);
+    auto bool_lanes = AllocateAligned<TI>(N);
+    auto lanes = AllocateAligned<T>(N);
+    HWY_ASSERT(bool_lanes && lanes);
+
+    const Vec<D> v = Iota(d, static_cast<T>(hwy::Unpredictable1() - 1));
+    Store(v, d, lanes.get());
+
+    const VI indices =
+        Reverse(di, Iota(di, static_cast<TI>(hwy::Unpredictable1() - 1)));
+
+    // Each lane should have a chance of having mask=true.
+    for (size_t rep = 0; rep < AdjustedReps(200); ++rep) {
+      for (size_t i = 0; i < N; ++i) {
+        bool_lanes[i] = (Random32(&rng) & 1024) ? TI(1) : TI(0);
+      }
+
+      const VI mask_i = Load(di, bool_lanes.get());
+      const auto mask = RebindMask(d, Gt(mask_i, Zero(di)));
+      const Vec<D> expected = IfThenElseZero(mask, Reverse(d, v));
+      const Vec<D> actual = MaskedGatherIndex(mask, d, lanes.get(), indices);
+      HWY_ASSERT_VEC_EQ(d, expected, actual);
+    }
+  }
+};
+
+HWY_NOINLINE void TestAllMaskedGather() {
+  ForUIF3264(ForPartialVectors<TestMaskedGather>());
+}
+
 struct TestBlendedStore {
   template <class T, class D>
   HWY_NOINLINE void operator()(T /*unused*/, D d) {
@@ -193,6 +276,8 @@ HWY_AFTER_NAMESPACE();
 namespace hwy {
 HWY_BEFORE_TEST(HwyMaskTest);
 HWY_EXPORT_AND_TEST_P(HwyMaskTest, TestAllMaskedLoad);
+HWY_EXPORT_AND_TEST_P(HwyMaskTest, TestAllMaskedScatter);
+HWY_EXPORT_AND_TEST_P(HwyMaskTest, TestAllMaskedGather);
 HWY_EXPORT_AND_TEST_P(HwyMaskTest, TestAllBlendedStore);
 HWY_EXPORT_AND_TEST_P(HwyMaskTest, TestAllStoreMaskBits);
 }  // namespace hwy
